﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Threading;
using System.Windows.Forms;

namespace LoginWithProgressBar
{
    public partial class TheForm : Form
    {
        // BackgroundWorker handles the lengthy operation leaving the UI thread responding
        private readonly BackgroundWorker _bw = new BackgroundWorker();
        
        // to keep the enabled/disabled controls together
        private List<Control> _enabledDisabledControls;

        public TheForm()
        {
            InitializeComponent();
        }

        private void BwRunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            // enable/disable controls
            ToggleControls();
        }

        private static void Login(object sender, DoWorkEventArgs doWorkEventArgs)
        {
            // emulate a lengthy process
            Thread.Sleep(3000);
        }

        private void ButtonLoginClick(object sender, EventArgs e)
        {
            // enable/disable controls
            ToggleControls();
            
            // async call to the lengthy operation to leave the UI thread responding
            _bw.RunWorkerAsync();
        }

        private void ToggleControls()
        {
            if (textBoxUserName.Enabled)
            {
                // show the progress bar during the lengthy operation
                progressBar.Show();
                
                // disable the controls
                foreach (Control control in _enabledDisabledControls)
                {
                    control.Enabled = false;
                }
            }
            else
            {
                progressBar.Hide();

                // enable the controls
                foreach (Control control in _enabledDisabledControls)
                {
                    control.Enabled = true;
                }
            }
        }

        private void TheForm_Load(object sender, EventArgs e)
        {
            // fill the controls List
            _enabledDisabledControls = new List<Control> {textBoxUserName, textBoxPassword, 
                labelUserName, labelPassword, buttonGo};

            // set the DoWork event handler
            _bw.DoWork += Login;

            // set the RunWorkerCompleted event handler
            _bw.RunWorkerCompleted += BwRunWorkerCompleted;
        }
    }
}